@@ -196,15 +196,15 @@ class SOCol(object):
         # alternateID means that this is a unique column that
         # can be used to identify rows
         self.alternateID = alternateID
-        if self.alternateID and alternateMethodName is None:
-            self.alternateMethodName = 'by' + self.name[0].capitalize() + self.name[1:]
-        else:
-            self.alternateMethodName = alternateMethodName
 
         if unique is NoDefault:
             self.unique = alternateID
         else:
             self.unique = unique
+        if self.unique and alternateMethodName is None:
+            self.alternateMethodName = 'by' + self.name[0].capitalize() + self.name[1:]
+        else:
+            self.alternateMethodName = alternateMethodName
 
         _validators = self.createValidators()
         if _validators:
@@ -354,7 +354,8 @@ class SOCol(object):
             # class attribute, return the descriptor itself
             return self
         if obj.sqlmeta._obsolete:
-            raise '@@: figure out the exception for a delete'
+            raise RuntimeError('The object <%s %s> is obsolete' % (
+                obj.__class__.__name__, obj.id))
         if obj.sqlmeta.cacheColumns:
             columns = obj.sqlmeta._columnCache
             if columns is None:
@@ -545,7 +546,7 @@ class UnicodeStringValidator(validators.Validator):
         if isinstance(value, unicode):
             return value
         if isinstance(value, array): # MySQL
-            value = value.tostring()
+            return unicode(value.tostring(), self.db_encoding)
         return unicode(value, self.db_encoding)
 
     def from_python(self, value, state):
@@ -975,7 +976,7 @@ class SOEnumCol(SOCol):
         return "VARCHAR(%i)" % (length), checkConstraint
 
     def _maxdbType(self):
-        raise "Enum type is not supported"
+        raise TypeError("Enum type is not supported on MAX DB")
 
     def _getlength(self, obj):
         """
@@ -1271,6 +1272,8 @@ class SOTimestampCol(SODateTimeCol):
     """
 
     def __init__(self, **kw):
+        if 'default' not in kw:
+            kw['default'] = None
         SOCol.__init__(self, **kw)
 
     def _mysqlType(self):
