@@ -1,28 +1,21 @@
 from sqlobject import sqlbuilder
 from sqlobject import classregistry
 from sqlobject.col import StringCol, ForeignKey
-from sqlobject.main import sqlmeta, SQLObject, SelectResults, True, False, \
+from sqlobject.main import sqlmeta, SQLObject, SelectResults, \
    makeProperties, unmakeProperties, getterName, setterName
+from sqlobject import events
 import iteration
 
-
-try:
-    basestring
-except NameError: # Python 2.2
-    import types
-    basestring = (types.StringType, types.UnicodeType)
-
-
-def tablesUsedDict(obj):
-    if hasattr(obj, "tablesUsedDict"):
-        return obj.tablesUsedDict()
-    elif isinstance(obj, (tuple, list)):
-        d = {}
+def tablesUsedSet(obj, db):
+    if hasattr(obj, "tablesUsedSet"):
+        return obj.tablesUsedSet(db)
+    elif isinstance(obj, (tuple, list, set, frozenset)):
+        s = set()
         for component in obj:
-            d.update(tablesUsedDict(component))
-        return d
+            s.update(tablesUsedSet(component, db))
+        return s
     else:
-        return {}
+        return set()
 
 
 class InheritableSelectResults(SelectResults):
@@ -32,26 +25,29 @@ class InheritableSelectResults(SelectResults):
             inheritedTables=None, **ops):
         if clause is None or isinstance(clause, str) and clause == 'all':
             clause = sqlbuilder.SQLTrueClause
-        tablesDict = tablesUsedDict(clause)
-        tablesDict[str(sourceClass.sqlmeta.table)] = 1
+
+        dbName = (ops.get('connection',None) or sourceClass._connection).dbName
+
+        tablesSet = tablesUsedSet(clause, dbName)
+        tablesSet.add(str(sourceClass.sqlmeta.table))
         orderBy = ops.get('orderBy')
         if inheritedTables:
             for tableName in inheritedTables:
-                tablesDict[str(tableName)] = 1
+                tablesSet.add(str(tableName))
         if orderBy and not isinstance(orderBy, basestring):
-            tablesDict.update(tablesUsedDict(orderBy))
+            tablesSet.update(tablesUsedSet(orderBy, dbName))
         #DSM: if this class has a parent, we need to link it
         #DSM: and be sure the parent is in the table list.
         #DSM: The following code is before clauseTables
         #DSM: because if the user uses clauseTables
         #DSM: (and normal string SELECT), he must know what he wants
         #DSM: and will do himself the relationship between classes.
-        if type(clause) is not str:
+        if not isinstance(clause, str):
             tableRegistry = {}
             allClasses = classregistry.registry(
                 sourceClass.sqlmeta.registry).allClasses()
             for registryClass in allClasses:
-                if str(registryClass.sqlmeta.table) in tablesDict:
+                if str(registryClass.sqlmeta.table) in tablesSet:
                     #DSM: By default, no parents are needed for the clauses
                     tableRegistry[registryClass] = registryClass
             tableRegistryCopy = tableRegistry.copy()
@@ -78,7 +74,7 @@ class InheritableSelectResults(SelectResults):
                     parentClass = currentClass.sqlmeta.parentClass
                     parentClause.append(currentClass.q.id == parentClass.q.id)
                     currentClass = parentClass
-                    tablesDict[str(currentClass.sqlmeta.table)] = 1
+                    tablesSet.add(str(currentClass.sqlmeta.table))
             clause = reduce(sqlbuilder.AND, parentClause, clause)
 
         super(InheritableSelectResults, self).__init__(sourceClass,
@@ -109,9 +105,16 @@ class InheritableSQLMeta(sqlmeta):
                 setattr(soClass, getterName(cname), eval(
                     'lambda self: self._parent.%s' % cname))
                 if not col.immutable:
-                    setattr(soClass, setterName(cname), eval(
-                        'lambda self, val: setattr(self._parent, %s, val)'
-                        % repr(cname)))
+                    def make_setfunc(cname):
+                        def setfunc(self, val):
+                            if not self.sqlmeta._creating and not getattr(self.sqlmeta, "row_update_sig_suppress", False):
+                                self.sqlmeta.send(events.RowUpdateSignal, self, {cname : val})
+                                
+                            result = setattr(self._parent, cname, val)
+                        return setfunc
+
+                    setfunc = make_setfunc(cname)
+                    setattr(soClass, setterName(cname), setfunc)
             if childUpdate:
                 makeProperties(soClass)
                 return
@@ -209,13 +212,14 @@ class InheritableSQLMeta(sqlmeta):
             columns.update(sm.parentClass.sqlmeta.columns)
             sm = sm.parentClass.sqlmeta
         return columns
+    getAllColumns = classmethod(getAllColumns)
 
-    def asDict(sqlmeta):
-        result = {}
-        for key in sqlmeta.getAllColumns():
-            result[key] = getattr(sqlmeta.instance, key)
-        result['id'] = sqlmeta.instance.id
-        return result
+    def getColumns(sqlmeta):
+        columns = sqlmeta.getAllColumns()
+        if columns.has_key('childName'):
+            del columns['childName']
+        return columns
+    getColumns = classmethod(getColumns)
 
 
 class InheritableSQLObject(SQLObject):
@@ -224,6 +228,12 @@ class InheritableSQLObject(SQLObject):
     _inheritable = True
     SelectResultsClass = InheritableSelectResults
 
+    def set(self, **kw):
+        if self._parent:
+            SQLObject.set(self, _suppress_set_sig=True, **kw)
+        else:
+            SQLObject.set(self, **kw)
+
     def __classinit__(cls, new_attrs):
         SQLObject.__classinit__(cls, new_attrs)
         # if we are a child class, add sqlbuilder fields from parents
@@ -232,7 +242,7 @@ class InheritableSQLObject(SQLObject):
             for column in currentClass.sqlmeta.columnDefinitions.values():
                 if column.name == 'childName':
                     continue
-                if type(column) == ForeignKey:
+                if isinstance(column, ForeignKey):
                     continue
                 setattr(cls.q, column.name,
                     getattr(currentClass.q, column.name))
