@@ -112,6 +112,16 @@ def test_selectBy_kwargs():
         return
     assert False, "IterTest(nonexistant='b') should raise TypeError"
 
+class UniqTest(SQLObject):
+    name = StringCol(dbName='name_col', unique=True)
+
+def test_by_uniq():
+    setupClass(UniqTest)
+    a = UniqTest(name='a')
+    b = UniqTest(name='b')
+    assert UniqTest.byName('a') is a
+    assert UniqTest.byName('b') is b
+
 class Counter2(SQLObject):
 
     n1 = IntCol(notNull=True)
@@ -151,8 +161,7 @@ def test_select_RLIKE():
     setupClass(IterTest)
 
     if IterTest._connection.dbName == "sqlite":
-        from sqlobject.sqlite import sqliteconnection
-        if not sqliteconnection.using_sqlite2:
+        if not IterTest._connection.using_sqlite2:
             return
 
         # Implement regexp() function for SQLite; only works with PySQLite2
@@ -180,8 +189,3 @@ def test_select_sqlbuilder():
     setupClass(IterTest)
     IterTest(name='sqlobject')
     IterTest.select(IterTest.q.name==u'sqlobject')
-
-def test_select_perConnection():
-    setupClass(IterTest)
-    IterTest(name='a')
-    assert not IterTest.select().getOne().sqlmeta._perConnection
